package utils;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

public class Config {
	
	private String arquivoConf = "validador.conf";
	private Properties defaultProps = new Properties();
	
	public Config() {
	  try {
			FileInputStream in = new FileInputStream(this.arquivoConf);
			this.defaultProps.load(in);
			in.close();
	  }
	  catch (FileNotFoundException fnf) {
		  this.criarConfig();
		  this.gravaConfig();
	  }
	  catch (IOException ex) {
		  System.out.println("Erro ao acessar arquivo "+this.arquivoConf);
	  }
	}
	
	public String getXml() {
		String path = this.defaultProps.getProperty("xml");
		if ((path == null) || (path.length() == 0))
			path = "";
		
		return path;
	}
	
	public void setXml(String path) {
		this.defaultProps.setProperty("xml", path);
		this.gravaConfig();
	}

	public String getXsd() {
		String path = this.defaultProps.getProperty("xsd");
		if ((path == null) || (path.length() == 0))
			path = "";

		return path;
	}

	public void setXsd(String path) {
		this.defaultProps.setProperty("xsd", path);
		this.gravaConfig();
	}

	/**
	 * Inicializa o arquivo de configuração, caso ele não seja encontrado no constructor
	 */
	private void criarConfig() {
  	try {
    	FileOutputStream out = new FileOutputStream(this.arquivoConf);
    	this.defaultProps.clear();
    	this.defaultProps.setProperty("xml", "arquivo.xml");
    	this.defaultProps.setProperty("xsd", "lotedms.xsd");
    	this.defaultProps.store(out, "Arquivo de configuracao criado automaticamente.\n"+
    			"Favor corrigir valores");
    	out.close();
		} catch (Exception e) {
			System.out.println("Erro ao criar arquivo "+this.arquivoConf);
		}
  }

	/**
	 * Salva valores das configurações
	 */
	private void gravaConfig() {
  	try {
    	FileOutputStream out = new FileOutputStream(this.arquivoConf);
    	this.defaultProps.store(out, "Arquivo de configuracao criado pelo sistema");
			out.close();
		} catch (Exception e) {
			System.out.println("Erro ao gravar arquivo "+this.arquivoConf);
		}
	}

}