package utils;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 *
 * @author alencar
 */
public class Validador {

    private File xml;
    private File xsd;
    private StringBuilder erros = new StringBuilder();
    private String nomeArquivoXML = "";
    private String nomeXSD = "";

    public Validador(String xml, String xsd) {
        this.xml = new File(xml);
        this.nomeArquivoXML = xml;

        this.xsd = new File(xsd);
        this.nomeXSD = xsd;
    }

    public boolean validar() {
        if (xml == null || !(xml.isFile() && xml.canRead())) {
            erros.append("Arquivo XML não encontrado ou não pode ser lido");
            return false;
        }

        if (xsd == null || !(xsd.isFile() && xsd.canRead())) {
            this.erros.append("Schema XML não encontrado ou não pode ser lido");
            return false;
        }
        return ValidaXML();
    }

    public StringBuilder getErros() {
        return this.erros;
    }

    /**
     * Valida o xml conforme o schema passado
     * @return true se o xml esta ok. False se contem erro
     */
    private boolean ValidaXML() {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        factory.setNamespaceAware(true);
        factory.setValidating(true);
        factory.setAttribute(
                "http://java.sun.com/xml/jaxp/properties/schemaLanguage", "http://www.w3.org/2001/XMLSchema");
        factory.setAttribute(
                "http://java.sun.com/xml/jaxp/properties/schemaSource", nomeXSD);
        DocumentBuilder builder = null;
        SimpleErrorHandler seh = new SimpleErrorHandler();

        try {
            builder = factory.newDocumentBuilder();
            builder.setErrorHandler(seh);
            if (seh.erro) {
                this.erros.append("Erro validando com schema (" + nomeXSD + ")\n");
                this.erros.append("Fim da validacao\n");
                return false;
            }
        } catch (Exception ex) {
            this.erros.append("Exception validando com schema (" + nomeXSD + ")\n");
            this.erros.append(ex.getMessage() + "\n");
            this.erros.append("Fim da validacao\n");
            return false;
        }

        try {
            String xml = "";

            try {
                FileReader reader = new FileReader(new File(this.nomeArquivoXML));
                BufferedReader leitor = new BufferedReader(reader);
                String linha = null;
                while (leitor.ready()) {
                    linha = leitor.readLine();
                    xml += linha + "\n";
                }
            } catch (Exception e) {
            }
            Document doc = builder.parse(new ByteArrayInputStream(xml.getBytes("UTF-8")));
            if (seh.erro) {
                this.erros.append("Erro validando xml (" + this.nomeArquivoXML + ") com schema (" + nomeXSD + "):\n");
                this.erros.append(seh.msg + "\n");
                this.erros.append("Fim da validacao\n");
                return false;
            }
        } catch (SAXException saxe) {
            this.erros.append("SAXException validando xml (" + this.nomeArquivoXML + ") com schema (" + nomeXSD + ")\n");
            this.erros.append(saxe.getMessage() + "\n");
            this.erros.append("Fim da validacao\n");
            return false;
        } catch (IOException ioe) {
            this.erros.append("IOException validando xml (" + this.nomeArquivoXML + ") com schema (" + nomeXSD + ")\n");
            this.erros.append(ioe.getMessage() + "\n");
            this.erros.append("Fim da validacao\n");
            return false;
        } catch (Exception se) {
            this.erros.append("Exception validando xml (" + this.nomeArquivoXML + ") com schema (" + nomeXSD + ")\n");
            this.erros.append(se.getMessage() + "\n");
            this.erros.append("Fim da validacao\n");
            return false;
        }

        return true;
    }
}
